/** @file ag_config_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.04.08

	@brief [ ag_config_dlg ]
*/

#include "ag_config_dlg.h"
#include <agar/core/types.h>
#include "../emu.h"
#include "../config.h"
#include "ag_gui_base.h"

AG_CONFIG_DLG::AG_CONFIG_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	memset(&param, 0, sizeof(param));
	param.fdd_type  = config.fdd_type;
	param.io_port = config.io_port;

	param.wav_reverse = 0;
	param.wav_half = 1;
	param.wav_correct = 0;
	param.wav_correct_type = 0;

	param.wav_sample_rate = 3;
	param.wav_sample_bits = 0;

	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;

	param.gl_filter_type = config.gl_filter_type;
	param.led_pos = config.led_pos;

	paramtmp = NULL;

	dirdlg = new AG_DIR_DLG(parent_emu, parent_gui);
}

AG_CONFIG_DLG::~AG_CONFIG_DLG()
{
	delete dirdlg;
	delete paramtmp;
}

/**
 * create config dialog
 */
void AG_CONFIG_DLG::Create()
{
	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;
	param.use_opengl = config.use_opengl;
	param.gl_filter_type = config.gl_filter_type;
	param.led_pos = config.led_pos;

	param.dipswitch = config.dipswitch;
	param.fdd_type = emu->get_parami(EMU::ParamFddType);
	param.io_port = emu->get_parami(EMU::ParamIOPort);
	param.wav_reverse = config.wav_reverse ? 1 : 0;
	param.wav_half = config.wav_half ? 1 : 0;
	param.wav_correct = config.wav_correct ? 1 : 0;
	param.wav_correct_type = config.wav_correct_type;
	param.wav_sample_rate = config.wav_sample_rate;
	param.wav_sample_bits = config.wav_sample_bits;

	memcpy(param.snap_path, config.snapshot_path, _MAX_PATH);
	memcpy(param.rom_path, config.rom_path, _MAX_PATH);
	memcpy(param.font_path, config.font_path, _MAX_PATH);
	memcpy(param.msg_font_name, config.msgboard_msg_fontname, _MAX_PATH);
	sprintf(param.msg_font_size, "%d", config.msgboard_msg_fontsize);
	memcpy(param.info_font_name, config.msgboard_info_fontname, _MAX_PATH);
	sprintf(param.info_font_size, "%d", config.msgboard_info_fontsize);
	memcpy(param.menu_font_name, config.menu_fontname, _MAX_PATH);
	sprintf(param.menu_font_size, "%d", config.menu_fontsize);

	for(int i=0; i<MAX_PRINTER; i++) {
		memcpy(param.prn_host[i], config.printer_server_host[i], _MAX_PATH);
		sprintf(param.prn_port[i], "%d", config.printer_server_port[i]);
	}
	for(int i=0; i<MAX_COMM; i++) {
		memcpy(param.com_host[i], config.comm_server_host[i], _MAX_PATH);
		sprintf(param.com_port[i], "%d", config.comm_server_port[i]);
		param.com_dipsw[i] = config.comm_dipswitch[i];
	}

	if (paramtmp == NULL) {
		paramtmp = new ConfigDlgParam;
	}
	*paramtmp = param;

	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE);
	AG_WindowSetCaptionS(win, "Configure");

	AG_Box *vbox_base;
	AG_Box *hbox_base;
	AG_Box *vbox_left;
	AG_Box *vbox_right;
	AG_Box *vbox;
	AG_Box *hbox;
//	AG_Label *lbl;
	AG_Radio *rad;
	AG_Checkbox *cbox;
	AG_UCombo *ucom;
	AG_Notebook *nb;
	AG_NotebookTab *tabs[5];

	int i;
	char tlen[40];
	for(i=0; i<sizeof(tlen); i++) tlen[i] = 'w';
	tlen[i-1]=0;

	nb = AG_NotebookNew(win, AG_NOTEBOOK_HFILL);

	//
	// 0 Mode
	tabs[0] = AG_NotebookAddTab(nb, "Mode", AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[0], AG_BOX_HFILL | AG_BOX_VFILL);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// left
	vbox_left = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// Mode switch
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "DIP switch (*)");

	strcpy(chkDipSwitchBase[0],"  MODE switch");
	if (config.dipswitch & 0x04) {
		chkDipSwitchBase[0][0] = '>';
	}
	cbox = AG_CheckboxNewFlag(vbox, 0, chkDipSwitchBase[0], (Uint *)&paramtmp->dipswitch, 0x04);

	// FDD type
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "FDD Type (*)");

	strcpy(radFddTypeBase[0],"  No FDD");
	strcpy(radFddTypeBase[1],"  Use 3inch compact FDD");
	strcpy(radFddTypeBase[2],"  Use 5.25inch mini FDD");
	for(i=0; i<3; i++) {
		radFddType[i] = radFddTypeBase[i];
		if (i == config.fdd_type) {
			radFddTypeBase[i][0] = '>';
		}
	}
	radFddType[3]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radFddType, &paramtmp->fdd_type);
	AG_AddEvent(rad, "radio-changed", OnChangeFddType, "%Cp", this);

	// right
	vbox_right = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// I/O Port address
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "I/O Port Address (*)");

	strcpy(chkIOPortBase[0],"  5.25inch FDC  $FF00 - $FF04");
	strcpy(chkIOPortBase[1],"  3inch FDC  $FF18 - $FF20");
	strcpy(chkIOPortBase[2],"  6voice PSG  $FF30 - $FF33");
	strcpy(chkIOPortBase[3],"  Ex LPT Port  $FF3C - $FF3F");
	strcpy(chkIOPortBase[4],"  Ex COM Port  $FF40 - $FF41");
	strcpy(chkIOPortBase[5],"  9voice PSG  $FF70 - $FF7F");
	strcpy(chkIOPortBase[6],"  KANJI ROM  $FF75 - $FF76");
	for(i=0; i<7; i++) {
		if (config.io_port & (1 << i)) {
			chkIOPortBase[i][0] = '>';
		}
		cbox = AG_CheckboxNewFlag(vbox, 0, chkIOPortBase[i], (Uint *)&paramtmp->io_port, (1 << i));
		AG_AddEvent(cbox, "checkbox-changed", OnChangeIOPort, "%Cp %i", this, i);
		chkIOPort[i] = cbox;
	}
	chkIOPort[7] = NULL;

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, "*  Need restart program or \"PowerOn\".");


	//
	// 1 screen
	tabs[1] = AG_NotebookAddTab(nb, "Screen", AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[1], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

#ifdef USE_OPENGL
	// GL Filter Type
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "OpenGL");

	const char *gl_use_list[] = {
		"No",
		"Yes(sync)",
		"Yes(async)",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Use (*)");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (gl_use_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, gl_use_list[i]);
	}
	AG_UComboSizeHint(ucom, "wwwwwwwwww", i);
	AG_TlistSelectText(ucom->list, gl_use_list[config.use_opengl]);
	AG_TlistSetChangedFn(ucom->list, OnChangeUseOpenGL, "%Cp", this);

	const char *gl_filter_list[] = {
		"Nearest",
		"Linear",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Filter Type");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (gl_filter_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, gl_filter_list[i]);
	}
	AG_UComboSizeHint(ucom, "wwwwwwwwww", i);
	AG_TlistSelectText(ucom->list, gl_filter_list[config.gl_filter_type]);
	AG_TlistSetChangedFn(ucom->list, OnChangeGLFilterType, "%Cp", this);
#endif

	// LED
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "LED");

	const char *led_pos_list[] = {
		"LeftTop",
		"RightTop",
		"LeftBottom",
		"RightBottom",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Position");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (led_pos_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, led_pos_list[i]);
	}
	AG_UComboSizeHint(ucom, "wwwwwwwwwwwwwww", i);
	AG_TlistSelectText(ucom->list, led_pos_list[config.led_pos]);
	AG_TlistSetChangedFn(ucom->list, OnChangeLedPos, "%Cp", this);

	// CRTC
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "CRTC");

	const char *skew_list[] = {
		"0",
		"1",
		"2",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Disptmg skew");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (skew_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, skew_list[i]);
	}
	AG_UComboSizeHint(ucom, "0000", i);
	AG_TlistSelectText(ucom->list, skew_list[config.disptmg_skew]);
	AG_TlistSetChangedFn(ucom->list, OnChangeDisptmgSkew, "%Cp", this);

	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Curdisp skew");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (skew_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, skew_list[i]);
	}
	AG_UComboSizeHint(ucom, "0000", i);
	AG_TlistSelectText(ucom->list, skew_list[config.curdisp_skew]);
	AG_TlistSetChangedFn(ucom->list, OnChangeCurdispSkew, "%Cp", this);

	// snapshot path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_Textbox *txtSnapPath = AG_TextboxNewS(hbox, AG_TEXTBOX_HFILL, "Snapshot path");
	AG_TextboxSizeHint(txtSnapPath, tlen);
	AG_TextboxBindUTF8(txtSnapPath, paramtmp->snap_path, sizeof(paramtmp->snap_path)-1);
	AG_ButtonNewFn(hbox, 0, "Folder...", OnShowDirDlg, "%Cp %Cp %i", this, win, SnapShotPath);

	// font path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_Textbox *txtFontPath = AG_TextboxNewS(hbox, AG_TEXTBOX_HFILL, "Font path");
	AG_TextboxSizeHint(txtFontPath, tlen);
	AG_TextboxBindUTF8(txtFontPath, paramtmp->font_path, sizeof(paramtmp->font_path)-1);
	AG_ButtonNewFn(hbox, 0, "Folder...", OnShowDirDlg, "%Cp %Cp %i", this, win, FontPath);

	// msgboard font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_Textbox *txtMsgFontNameS = AG_TextboxNewS(hbox, 0, "Message font");
	AG_TextboxSizeHint(txtMsgFontNameS, "wwwwwwwwwwwwwwww");
	AG_TextboxBindUTF8(txtMsgFontNameS, paramtmp->msg_font_name, sizeof(paramtmp->msg_font_name)-1);
	AG_Textbox *txtMsgFontSizeS = AG_TextboxNewS(hbox, 0, " size");
	AG_TextboxSizeHint(txtMsgFontSizeS, "wwww");
	AG_TextboxBindASCII(txtMsgFontSizeS, paramtmp->msg_font_size, sizeof(paramtmp->msg_font_size)-1);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_Textbox *txtMsgFontNameL = AG_TextboxNewS(hbox, 0, "Info font");
	AG_TextboxSizeHint(txtMsgFontNameL, "wwwwwwwwwwwwwwww");
	AG_TextboxBindUTF8(txtMsgFontNameL, paramtmp->info_font_name, sizeof(paramtmp->info_font_name)-1);
	AG_Textbox *txtMsgFontSizeL = AG_TextboxNewS(hbox, 0, " size");
	AG_TextboxSizeHint(txtMsgFontSizeL, "wwww");
	AG_TextboxBindASCII(txtMsgFontSizeL, paramtmp->info_font_size, sizeof(paramtmp->info_font_size)-1);

	// menu font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_Textbox *txtMenuFontName = AG_TextboxNewS(hbox, 0, "Menu font (*)");
	AG_TextboxSizeHint(txtMenuFontName, "wwwwwwwwwwwwwwww");
	AG_TextboxBindUTF8(txtMenuFontName, paramtmp->menu_font_name, sizeof(paramtmp->menu_font_name)-1);
	AG_Textbox *txtMenuFontSize = AG_TextboxNewS(hbox, 0, " size");
	AG_TextboxSizeHint(txtMenuFontSize, "wwww");
	AG_TextboxBindASCII(txtMenuFontSize, paramtmp->menu_font_size, sizeof(paramtmp->menu_font_size)-1);
	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, "*  Need restart program.");

	//
	// 2 tape
	tabs[2] = AG_NotebookAddTab(nb, "Tape", AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[2], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// Load wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "Load wav file");

	AG_CheckboxNewInt(vbox, 0, "reverse wave", &paramtmp->wav_reverse);
	AG_CheckboxNewInt(vbox, 0, "half wave", &paramtmp->wav_half);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	cbox = AG_CheckboxNewInt(hbox, 0, "correct", &paramtmp->wav_correct);

	const char *radCorrect[] = {
		"cos wave",
		"sin wave",
		NULL
	};
	rad = AG_RadioNewInt(hbox, AG_RADIO_EXPAND, radCorrect, &paramtmp->wav_correct_type);

	AG_AddEvent(cbox, "checkbox-changed", OnChangeCorrect, "%Cp %Cp", this, rad);
	if (paramtmp->wav_correct == 0) {
		AG_WidgetDisable(rad);
	} else {
		AG_WidgetEnable(rad);
	}

	// Save wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, "Save wav file");

	const char *rate_list[] = {
		"11025",
		"22050",
		"44100",
		"48000",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Sample rate");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (rate_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, rate_list[i]);
	}
	AG_UComboSizeHint(ucom, "0000000", i);
	AG_TlistSelectText(ucom->list, rate_list[paramtmp->wav_sample_rate]);
	AG_TlistSetChangedFn(ucom->list, OnChangeSampleRate, "%Cp", this);

	const char *bits_list[] = {
		"8",
		"16",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, "Sample bits");
	ucom = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
	for(i=0; ; i++) {
		if (bits_list[i] == NULL) break;
		AG_TlistAdd(ucom->list, NULL, bits_list[i]);
	}
	AG_UComboSizeHint(ucom, "0000", i);
	AG_TlistSelectText(ucom->list, bits_list[paramtmp->wav_sample_bits]);
	AG_TlistSetChangedFn(ucom->list, OnChangeSampleBits, "%Cp", this);

	//
	// 3 network
	tabs[3] = AG_NotebookAddTab(nb, "Network", AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[3], AG_BOX_EXPAND);

	// LPT
	for(i=0; i<MAX_PRINTER; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		AG_Textbox *txtLPT = AG_TextboxNew(hbox, 0, "LPT%d hostname", i);
		AG_TextboxSizeHint(txtLPT, "wwwwwwwwwwwwwww");
		AG_TextboxBindUTF8(txtLPT, paramtmp->prn_host[i], sizeof(paramtmp->prn_host[i])-1);
		AG_Textbox *txtLPTP = AG_TextboxNewS(hbox, 0, " port");
		AG_TextboxBindASCII(txtLPTP, paramtmp->prn_port[i], sizeof(paramtmp->prn_port[i])-1);
		AG_TextboxSizeHint(txtLPTP, "wwwwww");
	}
	// COM
	const char *comm_baud_list[] = {
		"S:300baud/F:1200baud",
		"S:600baud/F:2400baud",
		"S:1200baud/F:4800baud",
		"S:2400baud/F:9600baud",
		NULL
	};
	for(i=0; i<MAX_COMM; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		AG_Textbox *txtCOM = AG_TextboxNew(hbox, 0, "COM%d hostname", i);
		AG_TextboxSizeHint(txtCOM, "wwwwwwwwwwwwwww");
		AG_TextboxBindUTF8(txtCOM, paramtmp->com_host[i], sizeof(paramtmp->com_host[i])-1);
		AG_Textbox *txtCOMP = AG_TextboxNewS(hbox, 0, " port");
		AG_TextboxBindASCII(txtCOMP, paramtmp->com_port[i], sizeof(paramtmp->com_port[i])-1);
		AG_TextboxSizeHint(txtCOMP, "000000");

		AG_UCombo *ucomCOM = AG_UComboNew(hbox, AG_UCOMBO_HFILL);
		int j = 0;
		for(j=0; ; j++) {
			if (comm_baud_list[j] == NULL) break;
			AG_TlistAdd(ucomCOM->list, NULL, comm_baud_list[j]);
		}
		AG_UComboSizeHint(ucomCOM, "0000000000000000000000", j);
		AG_TlistSelectText(ucomCOM->list, comm_baud_list[paramtmp->com_dipsw[i]-1]);
		AG_TlistSetChangedFn(ucomCOM->list, OnChangeCommBaud, "%Cp %i", this, i);
	}

	//
	// 4 misc
	tabs[4] = AG_NotebookAddTab(nb, "Misc", AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[4], AG_BOX_EXPAND);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_Textbox *txtRomPath = AG_TextboxNewS(hbox, AG_TEXTBOX_HFILL, "ROM path");
	AG_TextboxSizeHint(txtRomPath, tlen);
	AG_TextboxBindUTF8(txtRomPath, paramtmp->rom_path, sizeof(paramtmp->rom_path)-1);
	AG_ButtonNewFn(hbox, 0, "Folder...", OnShowDirDlg, "%Cp %Cp %i", this, win, RomPath);
	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, "*  Need restart program.");

	//
	// button
	hbox_base = AG_BoxNewHoriz(win, AG_BOX_EXPAND);
	AG_ButtonNewFn(hbox_base, 0, "OK", OnOk, "%Cp %Cp", this, win);
	AG_ButtonNewFn(hbox_base, 0, "Cancel", OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);

	gui->Pause(true);
	AG_WindowShow(win);

	//
	change_fdd_type(paramtmp->fdd_type);
}

void AG_CONFIG_DLG::change_fdd_type(int index)
{
	if (index == 0) {
		paramtmp->io_port = paramtmp->io_port & ~3;
		AG_WidgetEnable(chkIOPort[0]);
		AG_WidgetEnable(chkIOPort[1]);
	} else if (index == 1) {
		paramtmp->io_port = (paramtmp->io_port & ~1) | 2;
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
	} else if (index == 2) {
		paramtmp->io_port = (paramtmp->io_port & ~2) | 1;
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
	}
}

void AG_CONFIG_DLG::change_io_port(int index, int status)
{
	if (index == 0 && status & (1 << 0)) {
		paramtmp->io_port &= ~(1 << 1);
	} else if (index == 1 && status & (1 << 1)) {
		paramtmp->io_port &= ~(1 << 0);
	}
	if (index == 5 && status & (1 << 5)) {
		paramtmp->io_port &= ~(1 << 6);
	} else if (index == 6 && status & (1 << 6)) {
		paramtmp->io_port &= ~(1 << 5);
	}
}

void AG_CONFIG_DLG::Close(AG_Window *win)
{
	AG_ObjectDetach(win);
	gui->Pause(false);
}

int AG_CONFIG_DLG::SetData(AG_Window *win)
{
	param = *paramtmp;

	config.disptmg_skew = param.disptmg_skew;
	config.curdisp_skew = param.curdisp_skew;
	config.use_opengl = (uint8)param.use_opengl;
	config.gl_filter_type = (uint8)param.gl_filter_type;
	config.led_pos = param.led_pos;

	config.dipswitch = (param.dipswitch & 0xff);

	emu->set_parami(EMU::ParamFddType, param.fdd_type);
	emu->set_parami(EMU::ParamIOPort, param.io_port);

	config.wav_reverse = (param.wav_reverse == 1);
	config.wav_half = (param.wav_half == 1);
	config.wav_correct = ( param.wav_correct == 1);
	config.wav_correct_type = (uint8)param.wav_correct_type;
	config.wav_sample_rate = (uint8)param.wav_sample_rate;
	config.wav_sample_bits = (uint8)param.wav_sample_bits;

	int val = 0;
	for(int i=0; i<MAX_PRINTER; i++) {
		strcpy(config.printer_server_host[i], param.prn_host[i]);
		val = 0;
		if (sscanf(param.prn_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.printer_server_port[i] = val;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		strcpy(config.comm_server_host[i], param.com_host[i]);
		val = 0;
		if (sscanf(param.com_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.comm_server_port[i] = val;
		}
		config.comm_dipswitch[i] = param.com_dipsw[i];
	}

	strcpy(config.snapshot_path, param.snap_path);
	strcpy(config.font_path, param.font_path);
	strcpy(config.msgboard_msg_fontname, param.msg_font_name);
	strcpy(config.msgboard_info_fontname, param.info_font_name);
	val = 0;
	if (sscanf(param.msg_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_msg_fontsize = val;
	}
	val = 0;
	if (sscanf(param.info_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_info_fontsize = val;
	}
	strcpy(config.menu_fontname, param.menu_font_name);
	val = 0;
	if (sscanf(param.menu_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.menu_fontsize = val;
	}

	strcpy(config.rom_path, param.rom_path);

	// set message font
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}
	// set menu font
//	gui->ConfigLoad();

	save_config();
	emu->change_pos_led(config.led_pos);
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif

	delete paramtmp;
	paramtmp = NULL;

	return 1;
}

void AG_CONFIG_DLG::show_dir_dlg(AG_Window *win, DirDlgType type)
{
	switch(type) {
	case RomPath:
		dirdlg->CreateLoad("Rom path", paramtmp->rom_path, sizeof(paramtmp->rom_path), win);
		break;
	case SnapShotPath:
		dirdlg->CreateLoad("Snapshot path", paramtmp->snap_path, sizeof(paramtmp->snap_path), win);
		break;
	case FontPath:
		dirdlg->CreateLoad("Font path", paramtmp->font_path, sizeof(paramtmp->font_path), win);
		break;
	}
}

/*
 * Event Handler (static)
 */
void AG_CONFIG_DLG::OnOk(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	if (dlg->SetData(win)) {
		dlg->Close(win);
	}
}

void AG_CONFIG_DLG::OnClose(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->Close(win);
}

void AG_CONFIG_DLG::OnChangeCorrect(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Radio *rad = (AG_Radio *)AG_PTR(2);

	if (dlg->paramtmp->wav_correct == 0) {
		AG_WidgetDisable(rad);
	} else {
		AG_WidgetEnable(rad);
	}
}

void AG_CONFIG_DLG::OnChangeSampleRate(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_rate = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeSampleBits(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_bits = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFddType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = (int)AG_INT(2);

	dlg->change_fdd_type(index);
}

void AG_CONFIG_DLG::OnChangeIOPort(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = AG_INT(2);
	int status = AG_INT(3);

	dlg->change_io_port(index, status);
}

void AG_CONFIG_DLG::OnChangeDisptmgSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->disptmg_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCurdispSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->curdisp_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeUseOpenGL(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->use_opengl = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeGLFilterType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->gl_filter_type = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeLedPos(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->led_pos = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCommBaud(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int num   = AG_INT(2);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(3);
	int state = AG_INT(4);

	if (state) {
		dlg->paramtmp->com_dipsw[num] = item->label + 1;
	}
}

void AG_CONFIG_DLG::OnShowDirDlg(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);
	DirDlgType type = (DirDlgType)AG_INT(3);

	dlg->show_dir_dlg(win, type);
}
