/** @file ag_gui.h

	Skelton for retropc emulator
	SDL + Agar edition

	@author Sasaji
	@date   2012.03.01

	@brief [ ag_gui ]
*/

#ifndef _AG_GUI_H_
#define _AG_GUI_H_

#include "ag_gui_base.h"

#define ID_CONFIGBOX	0
#define ID_KEYBINDBOX	1

#define EVT_OPEN_LOAD_RECKEY_DIALOG	1
#define EVT_OPEN_SAVE_RECKEY_DIALOG	2

class AG_CONFIG_DLG;
class AG_KEYBIND_DLG;

/**
	@brief gui vm dependency
*/
class AG_GUI: public AG_GUI_BASE {
public:
	AG_GUI(EMU *);
	virtual ~AG_GUI();

	// dialog
	AG_CONFIG_DLG *configbox;
	AG_KEYBIND_DLG *keybindbox;

	int InitCore();

	AG_DLG *GetDlgPtr(int);

	bool ExecuteGlobalKeys(int, Uint32);
	void ProcessUserEvent(int);

	void ToggleSyncIRQ(void);
	void ToggleStretchScreen(void);
	void ToggleCutoutScreen(void);
	void ChangeAfterImage(int num);
	void ChangeAfterImage(void);
	void ChangeKeepImage(int num);
	void ChangeKeepImage(void);
#ifdef USE_OPENGL
	void ChangeUseOpenGL(int num);
	void ChangeUseOpenGL(void);
#endif
	void ToggleLedBox(void);
	void ToggleMsgBoard(void);
	void ToggleUseJoypad(void);
	void CreateKeybindBox(void);
	void CreateConfigureBox(void);
	void LoadState(void);
	void SaveState(void);
	void LoadStateAndRecKey(void);
	void SaveStateAndRecKey(void);
	void PlayRecKey(void);
	void RecordRecKey(void);
	void RewindDataRec(void);
	void ShowLoadDataRecDialog(void);
	void ShowSaveDataRecDialog(void);
	void FastForwardDataRec(void);
	void ChangeFddType(int num);
	void ChangePositionLedBox(int num);

private:
	void set_menu_item(AG_Menu *);

	static void OnSelectSyncIRQ(AG_Event *);
	static void OnUpdateSyncIRQ(AG_Event *);
	static void OnSelectLoadState(AG_Event *);
	static void OnSelectSaveState(AG_Event *);
	static void OnSelectPlayRecKey(AG_Event *);
	static void OnUpdatePlayRecKey(AG_Event *);
	static void OnSelectStopPlayRecKey(AG_Event *);
	static void OnSelectRecordRecKey(AG_Event *);
	static void OnUpdateRecordRecKey(AG_Event *);
	static void OnSelectStopRecordRecKey(AG_Event *);
	static void OnSelectRewindDataRec(AG_Event *);
	static void OnSelectFastForwardDataRec(AG_Event *);
	static void OnSelectRealModeDataRec(AG_Event *);
	static void OnUpdateRealModeDataRec(AG_Event *);
	static void OnSelectChangeSideFloppyDisk(AG_Event *);
	static void OnUpdateChangeSideFloppyDisk(AG_Event *);
	static void OnSelectWriteProtectFloppyDisk(AG_Event *);
	static void OnUpdateWriteProtectFloppyDisk(AG_Event *);
	static void OnSelectStretchScreen(AG_Event *);
	static void OnUpdateStretchScreen(AG_Event *);
	static void OnSelectCutoutScreen(AG_Event *);
	static void OnUpdateCutoutScreen(AG_Event *);
	static void OnSelectAfterImage(AG_Event *);
	static void OnUpdateAfterImage(AG_Event *);
	static void OnSelectKeepImage(AG_Event *);
	static void OnUpdateKeepImage(AG_Event *);
#ifdef USE_OPENGL
	static void OnSelectUseOpenGL(AG_Event *);
	static void OnUpdateUseOpenGL(AG_Event *);
#endif
	static void OnSelectLedBox(AG_Event *);
	static void OnUpdateLedBox(AG_Event *);
	static void OnSelectMsgBoard(AG_Event *);
	static void OnUpdateMsgBoard(AG_Event *);
	static void OnSelectUseJoypad(AG_Event *);
	static void OnUpdateUseJoypad(AG_Event *);
	static void OnSelectEnableLightpen(AG_Event *);
	static void OnUpdateEnableLightpen(AG_Event *);
	static void OnSelectKeybindBox(AG_Event *);
	static void OnSelectConfigureBox(AG_Event *);
	static void OnSelectPrintPrinter(AG_Event *);
	static void OnUpdatePrintPrinter(AG_Event *);
	static void OnSelectDirectPrinter(AG_Event *);
	static void OnUpdateDirectPrinter(AG_Event *);
	static void OnSelectCommServer(AG_Event *);
	static void OnUpdateCommServer(AG_Event *);
	static void OnSelectCommConnect(AG_Event *);
	static void OnUpdateCommConnect(AG_Event *);
};

#endif /* _AG_GUI_H_ */
