/** @file bml3mk5_gui.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.3.3

	@brief [ bml3mk5_gui ]
*/

#include "ag_gui.h"
#include "ag_file_dlg.h"
#include "ag_config_dlg.h"
#include "ag_keybind_dlg.h"

AG_GUI::AG_GUI(EMU *parent_emu) : AG_GUI_BASE(parent_emu)
{
}

AG_GUI::~AG_GUI()
{
	delete keybindbox;
	delete configbox;
}

int AG_GUI::InitCore()
{
	int rc = 0;

	rc = AG_GUI_BASE::InitCore();
	if (rc != 0) return rc;

	// dialog
	configbox = new AG_CONFIG_DLG(emu, this);
	keybindbox = new AG_KEYBIND_DLG(emu, this);

	return rc;
}

AG_DLG *AG_GUI::GetDlgPtr(int id)
{
	AG_DLG *p = NULL;

	switch(id) {
		case ID_KEYBINDBOX:
			p = keybindbox;
			break;
		default:
			p = configbox;
			break;
	}
	return p;
}

/**
 * create menu bar
 */
void AG_GUI::set_menu_item(AG_Menu *menu)
{
	AG_MenuItem *mt_control;
	AG_MenuItem *mt_tape;
	AG_MenuItem *mt_fdd;
	AG_MenuItem *mt_screen;
	AG_MenuItem *mt_sound;
	AG_MenuItem *mt_printer;
	AG_MenuItem *mt_comm;
	AG_MenuItem *mt_options;
	AG_MenuItem *mt_help;

	AG_MenuItem *mi;
	AG_MenuItem *ms;

	// create menu
	mt_control = AG_MenuNode(menu->root, "Control", NULL);
	{
		//                              **--------------- ------
		mi = AG_MenuAction(mt_control, "  PowerOn         Alt+F3", NULL, OnSelectReset, "%Cp", this);
		mi = AG_MenuAction(mt_control, "  MODE Switch     Alt+M", NULL, OnSelectDipswitch, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateDipswitch, "%Cp %i", this, 2);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, "  Reset Switch    Alt+R", NULL, OnSelectWarmReset, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, "  Pause           Alt+Q", NULL, OnSelectPause, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdatePause, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   "  CPU Power", NULL);
			ms = AG_MenuAction(mi,     "  CPU x0.5        Alt+9", NULL, OnSelectCPUPower, "%Cp %i", this, 0);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 0);
			ms = AG_MenuAction(mi,     "  CPU x1          Alt+1", NULL, OnSelectCPUPower, "%Cp %i", this, 1);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,     "  CPU x2          Alt+2", NULL, OnSelectCPUPower, "%Cp %i", this, 2);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 2);
			ms = AG_MenuAction(mi,     "  CPU x4          Alt+3", NULL, OnSelectCPUPower, "%Cp %i", this, 3);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 3);
			ms = AG_MenuAction(mi,     "  CPU x8          Alt+4", NULL, OnSelectCPUPower, "%Cp %i", this, 4);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 4);
			ms = AG_MenuAction(mi,     "  CPU x16         Alt+5", NULL, OnSelectCPUPower, "%Cp %i", this, 5);
				 AG_MenuSetPollFn(ms, OnUpdateCPUPower, "%Cp %i", this, 5);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, "  Sync IRQ        Alt+0", NULL, OnSelectSyncIRQ, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateSyncIRQ, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   "  Auto key", NULL);
			ms = AG_MenuAction(mi,     "  Open...", NULL, OnSelectStartAutoKey, "%Cp %s"
				, this, "*.txt,*.bas,*.lpt");
				 AG_MenuSetPollFn(ms, OnUpdateStartAutoKey, "%Cp", this);
			ms = AG_MenuAction(mi    , "  Stop", NULL, OnSelectStopAutoKey, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuNode(mt_control,   "  Record key", NULL);
			ms = AG_MenuAction(mi,     "  Play...", NULL, OnSelectPlayRecKey, "%Cp"
				, this);
				 AG_MenuSetPollFn(ms, OnUpdatePlayRecKey, "%Cp", this);
			ms = AG_MenuAction(mi    , "  Stop play", NULL, OnSelectStopPlayRecKey, "%Cp", this);
			AG_MenuSeparator(mi);
			ms = AG_MenuAction(mi,     "  Rec...", NULL, OnSelectRecordRecKey, "%Cp"
				, this);
				 AG_MenuSetPollFn(ms, OnUpdateRecordRecKey, "%Cp", this);
			ms = AG_MenuAction(mi    , "  Stop rec", NULL, OnSelectStopRecordRecKey, "%Cp", this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, "  Load State...   Alt+O", NULL, OnSelectLoadState, "%Cp"
			, this);
		mi = AG_MenuAction(mt_control, "  Save State...", NULL, OnSelectSaveState, "%Cp"
			, this);
		AG_MenuSeparator(mt_control);
		mi = AG_MenuAction(mt_control, "  Exit            Alt+F4", NULL, OnSelectExit, "%Cp", this);
	}
	mt_tape = AG_MenuNode(menu->root, "Tape", NULL);
	{
		//                           **--------------- ------
		mi = AG_MenuAction(mt_tape, "  Play...         Alt+F7", NULL, OnSelectLoadDataRec, "%Cp"
			, this);
			 AG_MenuSetPollFn(mi, OnUpdateLoadDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, "  Rec...          Alt+F6", NULL, OnSelectSaveDataRec, "%Cp"
			, this);
			 AG_MenuSetPollFn(mi, OnUpdateSaveDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, "  Rewind          Alt+F5", NULL, OnSelectRewindDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, "  F.F.            Alt+F8", NULL, OnSelectFastForwardDataRec, "%Cp", this);
		mi = AG_MenuAction(mt_tape, "  Eject", NULL, OnSelectCloseDataRec, "%Cp", this);
		AG_MenuSeparator(mt_tape);
		mi = AG_MenuAction(mt_tape, "  Real mode", NULL, OnSelectRealModeDataRec, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateRealModeDataRec, "%Cp", this);
		AG_MenuSeparator(mt_tape);
		mi = AG_MenuDynamicItem(mt_tape, "  Recent", NULL, OnUpdateRecent, "%Cp %i %i", this, AG_FILE_DLG::DATAREC, 0);
	}
	for(int drv=0; drv<2; drv++) {
		char name[32];
		sprintf(name,"FDD%d",drv);
		mt_fdd = AG_MenuNode(menu->root, name, NULL);
		{
		//                              **--------------- ------
			sprintf(name,              "  Open...         Alt+F%d", drv + 9);
			mi = AG_MenuAction(mt_fdd, name, NULL, OnSelectOpenFloppyDisk, "%Cp %i"
				, this, drv);
				 AG_MenuSetPollFn(mi, OnUpdateOpenFloppyDisk, "%Cp %i", this, drv);
			mi = AG_MenuAction(mt_fdd, "  Change side to A", NULL, OnSelectChangeSideFloppyDisk, "%Cp %i"
				, this, drv);
				 AG_MenuSetPollFn(mi, OnUpdateChangeSideFloppyDisk, "%Cp %i", this, drv);
			mi = AG_MenuAction(mt_fdd, "  Close", NULL, OnSelectCloseFloppyDisk, "%Cp %i", this, drv);
			AG_MenuSeparator(mt_fdd);
			mi = AG_MenuAction(mt_fdd, "  Write protect", NULL, OnSelectWriteProtectFloppyDisk, "%Cp %i", this, drv);
				 AG_MenuSetPollFn(mi, OnUpdateWriteProtectFloppyDisk, "%Cp %i", this, drv);
			AG_MenuSeparator(mt_fdd);
			mi = AG_MenuDynamicItem(mt_fdd, "  Multi Volume", NULL, OnUpdateMultiVolumeList, "%Cp %i", this, drv);
			AG_MenuSeparator(mt_fdd);
			mi = AG_MenuDynamicItem(mt_fdd, "  Recent", NULL, OnUpdateRecent, "%Cp %i %i", this, AG_FILE_DLG::FLOPPY, drv);
		}
	}
	mt_screen = AG_MenuNode(menu->root, "Screen", NULL);
	{
		//                             **--------------- ------
		mi = AG_MenuNode(mt_screen,   "  Frame rate", NULL);
			ms = AG_MenuAction(mi,    "  Auto", NULL, OnSelectFrameRate, "%Cp %i", this, -1);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, -1);
			ms = AG_MenuAction(mi,    "  60fps", NULL, OnSelectFrameRate, "%Cp %i", this, 0);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 0);
			ms = AG_MenuAction(mi,    "  30fps", NULL, OnSelectFrameRate, "%Cp %i", this, 1);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 1);
			ms = AG_MenuAction(mi,    "  20fps", NULL, OnSelectFrameRate, "%Cp %i", this, 2);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 2);
			ms = AG_MenuAction(mi,    "  15fps", NULL, OnSelectFrameRate, "%Cp %i", this, 3);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 3);
			ms = AG_MenuAction(mi,    "  12fps", NULL, OnSelectFrameRate, "%Cp %i", this, 4);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 4);
			ms = AG_MenuAction(mi,    "  10fps", NULL, OnSelectFrameRate, "%Cp %i", this, 5);
				 AG_MenuSetPollFn(ms, OnUpdateFrameRate, "%Cp %i", this, 5);
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuNode(mt_screen,   "  Record", NULL);
		{
			char str[100];
			for(int i = 0; i < 2; i++) {
				emu->get_screen_record_size_str(i, str);
				ms = AG_MenuAction(mi, str, NULL, OnSelectScreenRecordSize, "%Cp %i", this, i);
					 AG_MenuSetPollFn(ms, OnUpdateScreenRecordSize, "%Cp %i", this, i);
			}
			AG_MenuSeparator(mi);
			ms = AG_MenuAction(mi,    "  Capture", NULL, OnSelectScreenCapture, "%Cp", this);
		}
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuNode(mt_screen,   "  Window", NULL);
		for(int i = 0; i < emu->get_window_mode_count(); i++) {
			char str[100];
			emu->get_window_mode_str(i, str);
			ms = AG_MenuAction(mi, str, NULL, OnSelectWindowMode, "%Cp %i", this, i);
				 AG_MenuSetPollFn(ms, OnUpdateWindowMode, "%Cp %i", this, i);
		}
		mi = AG_MenuNode(mt_screen,   "  Fullscreen", NULL);
			ms = AG_MenuAction(mi, "  Stretch screen", NULL, OnSelectStretchScreen, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateStretchScreen, "%Cp", this);
			ms = AG_MenuAction(mi, "  Cutout screen", NULL, OnSelectCutoutScreen, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateCutoutScreen, "%Cp", this);
			AG_MenuSeparator(mi);
		for(int i = 0; i < emu->get_screen_mode_count(); i++) {
			char str[100];
			emu->get_screen_mode_str(i, str);
			ms = AG_MenuAction(mi, str, NULL, OnSelectScreenMode, "%Cp %i", this, i);
				 AG_MenuSetPollFn(ms, OnUpdateScreenMode, "%Cp %i", this, i);
		}
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, "  Scanline        Alt+S", NULL, OnSelectScanLine, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateScanLine, "%Cp", this);
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, "  Afterimage1     Alt+T", NULL, OnSelectAfterImage, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateAfterImage, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_screen, "  Afterimage2     Alt+T", NULL, OnSelectAfterImage, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateAfterImage, "%Cp %i", this, 2);
		mi = AG_MenuAction(mt_screen, "  Keepimage1", NULL, OnSelectKeepImage, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateKeepImage, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_screen, "  Keepimage2", NULL, OnSelectKeepImage, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateKeepImage, "%Cp %i", this, 2);
#ifdef USE_OPENGL
		AG_MenuSeparator(mt_screen);
		mi = AG_MenuAction(mt_screen, "  UseOpenGL(Sync) Alt+Y", NULL, OnSelectUseOpenGL, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateUseOpenGL, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_screen, "  UseOpenGL(Async)Alt+Y", NULL, OnSelectUseOpenGL, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateUseOpenGL, "%Cp %i", this, 2);
#endif
	}
	mt_sound = AG_MenuNode(menu->root, "Sound", NULL);
	{
		//                            **--------------- ------
		mi = AG_MenuAction(mt_sound, "  Volume...       Alt+V", NULL, OnSelectSoundVolume, "%Cp", this);
		AG_MenuSeparator(mt_sound);
		mi = AG_MenuNode(mt_sound,   "  Record", NULL);
		{
			ms = AG_MenuAction(mi, "  Rec", NULL, OnSelectSoundStartRecord, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateSoundStartRecord, "%Cp", this);
			ms = AG_MenuAction(mi, "  Stop", NULL, OnSelectSoundStopRecord, "%Cp", this);
				 AG_MenuSetPollFn(ms, OnUpdateSoundStopRecord, "%Cp", this);
		}
		AG_MenuSeparator(mt_sound);
//		mi = AG_MenuAction(mt_sound, "  2000Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 0);
//			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 0);
//		mi = AG_MenuAction(mt_sound, "  4000Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 1);
//			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_sound, "  8000Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 2);
		mi = AG_MenuAction(mt_sound, "  11025Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 3);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 3);
		mi = AG_MenuAction(mt_sound, "  22050Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 4);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 4);
		mi = AG_MenuAction(mt_sound, "  44100Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 5);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 5);
		mi = AG_MenuAction(mt_sound, "  48000Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 6);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 6);
		mi = AG_MenuAction(mt_sound, "  96000Hz", NULL, OnSelectSoundRate, "%Cp %i", this, 7);
			 AG_MenuSetPollFn(mi, OnUpdateSoundRate, "%Cp %i", this, 7);
		AG_MenuSeparator(mt_sound);
		mi = AG_MenuAction(mt_sound, "  50msec", NULL, OnSelectSoundLatency, "%Cp %i", this, 0);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 0);
		mi = AG_MenuAction(mt_sound, "  100msec", NULL, OnSelectSoundLatency, "%Cp %i", this, 1);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 1);
		mi = AG_MenuAction(mt_sound, "  200msec", NULL, OnSelectSoundLatency, "%Cp %i", this, 2);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 2);
		mi = AG_MenuAction(mt_sound, "  300msec", NULL, OnSelectSoundLatency, "%Cp %i", this, 3);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 3);
		mi = AG_MenuAction(mt_sound, "  400msec", NULL, OnSelectSoundLatency, "%Cp %i", this, 4);
			 AG_MenuSetPollFn(mi, OnUpdateSoundLatency, "%Cp %i", this, 4);
	}
	mt_printer = AG_MenuNode(menu->root, "Printer", NULL);
	{
		for(int drv = 0; drv < MAX_PRINTER; drv++) {
			char name[20];
			sprintf(name,"  LPT%d", drv);
			mi = AG_MenuNode(mt_printer, name, NULL);
			{
				ms = AG_MenuAction(mi, "  Save...", NULL, OnSelectSavePrinter, "%Cp %i %s"
					, this, drv, "*.lpt");
					 AG_MenuSetPollFn(ms, OnUpdateSavePrinter, "%Cp %i", this, drv);
				ms = AG_MenuAction(mi, "  Printer", NULL, OnSelectPrintPrinter, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdatePrintPrinter, "%Cp %i", this, drv);
				ms = AG_MenuAction(mi, "  Clear", NULL, OnSelectClearPrinter, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateClearPrinter, "%Cp %i", this, drv);
				AG_MenuSeparator(mi);
				ms = AG_MenuAction(mi, "  Direct", NULL, OnSelectDirectPrinter, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateDirectPrinter, "%Cp %i", this, drv);
			}
		}
	}
	mt_comm = AG_MenuNode(menu->root, "Comm", NULL);
	{
		for(int drv = 0; drv < MAX_COMM; drv++) {
			char name[20];
			sprintf(name,"  COM%d", drv);
			mi = AG_MenuNode(mt_comm, name, NULL);
			{
				ms = AG_MenuAction(mi, "  Enable server", NULL, OnSelectCommServer, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateCommServer, "%Cp %i", this, drv);
				ms = AG_MenuAction(mi, "  Connect", NULL, OnSelectCommConnect, "%Cp %i", this, drv);
					 AG_MenuSetPollFn(ms, OnUpdateCommConnect, "%Cp %i", this, drv);
			}
		}
	}
	mt_options = AG_MenuNode(menu->root, "Options", NULL);
	{
		//                              **--------------- ------
		mi = AG_MenuAction(mt_options, "  Show LED        Alt+L", NULL, OnSelectLedBox, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateLedBox, "%Cp", this);
		mi = AG_MenuAction(mt_options, "  Show Message    Alt+Z", NULL, OnSelectMsgBoard, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateMsgBoard, "%Cp", this);
		AG_MenuSeparator(mt_options);
		mi = AG_MenuAction(mt_options, "  Use Joypad", NULL, OnSelectUseJoypad, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateUseJoypad, "%Cp", this);
		mi = AG_MenuAction(mt_options, "  Enable Lightpen", NULL, OnSelectEnableLightpen, "%Cp", this);
			 AG_MenuSetPollFn(mi, OnUpdateEnableLightpen, "%Cp", this);
		mi = AG_MenuAction(mt_options, "  Keybind...      Alt+K", NULL, OnSelectKeybindBox, "%Cp", this);
		AG_MenuSeparator(mt_options);
		mi = AG_MenuAction(mt_options, "  Configure...    Alt+C", NULL, OnSelectConfigureBox, "%Cp", this);
	}
	mt_help = AG_MenuNode(menu->root, "Help", NULL);
	{
		mi = AG_MenuAction(mt_help, "  About...", NULL, OnSelectAbout, "%Cp", this);
	}
}

/**
 * global keys
 *
 * @return true : caller move to gui event loop to show dialog.
 */
bool AG_GUI::ExecuteGlobalKeys(int key, Uint32 mod)
{
	bool need_gui = false;

//	if (key != AG_KEY_LALT && key != AG_KEY_RALT && (mod & (AG_KEYMOD_LALT | AG_KEYMOD_RALT))) {
		// Alt + ...
		switch(key) {
			case '0':	// sync IRQ
				ToggleSyncIRQ();
				break;
			case '1':	// CPU x1
				CPUPower(1);
				break;
			case '2':	// CPU x2
				CPUPower(2);
				break;
			case '3':	// CPU x4
				CPUPower(3);
				break;
			case '4':	// CPU x8
				CPUPower(4);
				break;
			case '5':	// CPU x16
				CPUPower(5);
				break;
			case '9':	// CPU x0.5
				CPUPower(0);
				break;
			case 'C':	// configure...
				CreateConfigureBox();
				need_gui = true;
				break;
			case 'E':	// load record key...
				PlayRecKey();
				need_gui = true;
				break;
			case 'F':	// change fdd mode
				ChangeFddType(-1);
				break;
			case 'K':	// key bind...
				CreateKeybindBox();
				need_gui = true;
				break;
			case 'L':	// show ledbox
				ToggleLedBox();
				break;
			case 'M':	// Mode switch
				Dipswitch(2);
				break;
			case 'O':	// load state...
				LoadState();
				need_gui = true;
				break;
			case 'P':	// pos change ledbox
				ChangePositionLedBox(-1);
				break;
			case 'Q':	// pause
				Pause();
				break;
			case 'R':	// reset switch
				WarmReset();
				break;
			case 'S':	// scanline
				ToggleScanLine();
				break;
			case 'T':	// afterimage1,2
				ChangeAfterImage();
				break;
			case 'V':	// volume
				CreateVolumeBox();
				need_gui = true;
				break;
#ifdef USE_OPENGL
			case 'Y':	// use opengl1,2
				ChangeUseOpenGL();
				break;
#endif
			case 'Z':	// show Message
				ToggleMsgBoard();
				break;
			case AG_KEY_F3:	// PowerOn
				Reset();
				break;
			case AG_KEY_F4:	// exit
				Exit();
				break;
			case AG_KEY_F5:	// Rewind
				RewindDataRec();
				break;
			case AG_KEY_F6:	// Rec...
				ShowSaveDataRecDialog();
				need_gui = true;
				break;
			case AG_KEY_F7:	// Play...
				ShowLoadDataRecDialog();
				need_gui = true;
				break;
			case AG_KEY_F8:	// F.F.
				FastForwardDataRec();
				break;
			case AG_KEY_F9:	// Open FDD0
				ShowOpenFloppyDiskDialog(0);
				need_gui = true;
				break;
			case AG_KEY_F10:	// Open FDD1
				ShowOpenFloppyDiskDialog(1);
				need_gui = true;
				break;
			case AG_KEY_RETURN:	// screen mode
				ChangeWindowMode(-1);
				break;
		}
//	}
	return need_gui;
}

/**
 * user event
 */
void AG_GUI::ProcessUserEvent(int id)
{
	switch(id) {
	case EVT_OPEN_LOAD_RECKEY_DIALOG:
		PlayRecKey();
		break;
	case EVT_OPEN_SAVE_RECKEY_DIALOG:
		RecordRecKey();
		break;
	}
}

/****************************************
 * Event Operation
 */
void AG_GUI::ToggleSyncIRQ(void)
{
	emu->change_sync_irq();
}
// Change Stretch Screen
void AG_GUI::ToggleStretchScreen(void)
{
	emu->change_stretch_screen();
}
// Change Cutoff Screen
void AG_GUI::ToggleCutoutScreen(void)
{
	emu->change_cutout_screen();
}
// Change After Image
void AG_GUI::ChangeAfterImage(int num)
{
	emu->change_screen_afterimage(num);
}
void AG_GUI::ChangeAfterImage(void)
{
	emu->change_screen_afterimage(-1);
}
// Change Keep Image
void AG_GUI::ChangeKeepImage(int num)
{
	emu->change_screen_keepimage(num);
}
void AG_GUI::ChangeKeepImage(void)
{
	emu->change_screen_keepimage(-1);
}
#ifdef USE_OPENGL
// Change Use OpenGL
void AG_GUI::ChangeUseOpenGL(int num)
{
	emu->change_screen_use_opengl(num);
}
void AG_GUI::ChangeUseOpenGL(void)
{
	emu->change_screen_use_opengl(-1);
}
#endif
// Change Ledbox
void AG_GUI::ToggleLedBox(void)
{
	emu->show_led();
}
// Change MsgBoard
void AG_GUI::ToggleMsgBoard(void)
{
	emu->show_message_board();
}
// Change UseJoypad
void AG_GUI::ToggleUseJoypad(void)
{
	emu->toggle_use_joypad();
}
// create keybind dialog
void AG_GUI::CreateKeybindBox(void)
{
	AG_WindowShow(now_driver->win);
	keybindbox->Create();
}
// create config dialog
void AG_GUI::CreateConfigureBox(void)
{
	AG_WindowShow(now_driver->win);
	configbox->Create();
}
// load State
void AG_GUI::LoadState(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateLoad(AG_FILE_DLG::STATUS, "Load Status Data"
		, "*.l3r", config.initial_state_path, NULL);
}
// save State
void AG_GUI::SaveState(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateSave(AG_FILE_DLG::STATUS, "Save Status Data"
		, "*.l3r", config.initial_state_path, NULL);
}
// load State and RecordKey
void AG_GUI::LoadStateAndRecKey(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateLoad(AG_FILE_DLG::STATUS_AND_RECKEY, "Load Status Data"
		, "*.l3r", config.initial_state_path, NULL);
}
// save State and RecordKey
void AG_GUI::SaveStateAndRecKey(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateSave(AG_FILE_DLG::STATUS_AND_RECKEY, "Save Status Data"
		, "*.l3r", config.initial_state_path, NULL);
}
// Play RecordKey
void AG_GUI::PlayRecKey(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateLoad(AG_FILE_DLG::RECKEY, "Play recorded keys"
		, "*.l3k", config.initial_state_path, NULL);
}
// Record RecordKey
void AG_GUI::RecordRecKey(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateSave(AG_FILE_DLG::RECKEY, "Record input keys"
		, "*.l3k", config.initial_state_path, NULL);
}
// rewind Data Recorder file
void AG_GUI::RewindDataRec(void)
{
	emu->rewind_datarec();
}
// show Data Recorder Load Dialog
void AG_GUI::ShowLoadDataRecDialog(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateLoad(AG_FILE_DLG::DATAREC, "Play Data Recorder Tape"
		, "*.l3,*.l3b,*.l3c,*.wav,*.t9x", config.initial_datarec_path, NULL);
}
// show Data Recorder Save Dialog
void AG_GUI::ShowSaveDataRecDialog(void)
{
	AG_WindowShow(now_driver->win);
	filebox->CreateSave(AG_FILE_DLG::DATAREC, "Record Data Recorder Tape"
		, "*.l3,*.l3b,*.l3c,*.wav,*.t9x", config.initial_datarec_path, NULL);
}
// Fast-Forward Data Recorder file
void AG_GUI::FastForwardDataRec(void)
{
	emu->fast_forward_datarec();
}
// change FDD Type
void AG_GUI::ChangeFddType(int num)
{
	emu->change_fdd_type(num, false);
}
// change LedBox position
void AG_GUI::ChangePositionLedBox(int num)
{
	emu->change_pos_led(num);
}

/*
 * Event Handler (static)
 */
void AG_GUI::OnSelectSyncIRQ(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleSyncIRQ();
}
// update sync irq
void AG_GUI::OnUpdateSyncIRQ(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, config.sync_irq);
}

// Show Status Data Load Dialog
void AG_GUI::OnSelectLoadState(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->LoadState();
}
// Show Status Data Save Dialog
void AG_GUI::OnSelectSaveState(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->SaveState();
}

// Show Play RecordKey Dialog
void AG_GUI::OnSelectPlayRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->PlayRecKey();
}
// update Play RecordKey
void AG_GUI::OnUpdatePlayRecKey(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, config.reckey_playing);
}
// stop Play RecordKey
void AG_GUI::OnSelectStopPlayRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->emu->stop_reckey(true, false);
}

// Show Record RecordKey Dialog
void AG_GUI::OnSelectRecordRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
//	char *ext = AG_STRING(2);

	gui->SaveStateAndRecKey();
}
// update Record RecordKey
void AG_GUI::OnUpdateRecordRecKey(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, config.reckey_recording);
}
// stop Play RecordKey
void AG_GUI::OnSelectStopRecordRecKey(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->emu->stop_reckey(false, true);
}

// update Rewind Data Recorder file
void AG_GUI::OnSelectRewindDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->RewindDataRec();
}
// update Fast-Forward Data Recorder file
void AG_GUI::OnSelectFastForwardDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->FastForwardDataRec();
}
// update Realmode Data Recorder
void AG_GUI::OnSelectRealModeDataRec(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->emu->realmode_datarec();
}
// update Realmode Data Recorder
void AG_GUI::OnUpdateRealModeDataRec(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, config.realmode_datarec);
}

// Show Floppy Disk Change Side
void AG_GUI::OnSelectChangeSideFloppyDisk(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	gui->emu->change_disk(drv);
}
// update Floppy Disk Change Side
void AG_GUI::OnUpdateChangeSideFloppyDisk(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	int side = gui->emu->get_disk_side(drv);

	if ((mi) && (mi)->text) {
		if (side) {
			// ..Change side to A
			(mi)->text[17] = 'A';
		} else {
			(mi)->text[17] = 'B';
		}
	}
	AG_GUI_MENU_ENABLE(mi, config.fdd_type != 0);
}

// Show Floppy Disk Write Protect
void AG_GUI::OnSelectWriteProtectFloppyDisk(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	gui->emu->toggle_disk_write_protect(drv);
}
// update Floppy Disk Write Protect
void AG_GUI::OnUpdateWriteProtectFloppyDisk(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int   drv = AG_INT(2);

	AG_GUI_MENU_ENABLE(mi, gui->emu->disk_inserted(drv));
	AG_GUI_MENU_CHECK(mi, gui->emu->disk_write_protected(drv));
}

// Change Stretch screen
void AG_GUI::OnSelectStretchScreen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->ToggleStretchScreen();
}
// update StretchScreen
void AG_GUI::OnUpdateStretchScreen(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, config.stretch_screen);
}

// Change Cutout screen
void AG_GUI::OnSelectCutoutScreen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	gui->ToggleCutoutScreen();
}
// update CutoutScreen
void AG_GUI::OnUpdateCutoutScreen(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();

	AG_GUI_MENU_CHECK(mi, config.cutout_screen);
}

// Change After Image
void AG_GUI::OnSelectAfterImage(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);
	gui->ChangeAfterImage(num);
}
// update After Image
void AG_GUI::OnUpdateAfterImage(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, config.afterimage == num);
}

// Change Keep Image
void AG_GUI::OnSelectKeepImage(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);
	gui->ChangeKeepImage(num);
}
// update Keep Image
void AG_GUI::OnUpdateKeepImage(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, config.keepimage == num);
}
#ifdef USE_OPENGL
// Change Use OpenGL
void AG_GUI::OnSelectUseOpenGL(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int num = AG_INT(2);
	gui->ChangeUseOpenGL(num);
}
// update Use OpenGL
void AG_GUI::OnUpdateUseOpenGL(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	int num = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, config.use_opengl == num);
}
#endif

// Change Ledbox
void AG_GUI::OnSelectLedBox(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleLedBox();
}
// update Ledbox
void AG_GUI::OnUpdateLedBox(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->emu->is_shown_ledbox(0));
}
// Change MsgBoard
void AG_GUI::OnSelectMsgBoard(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleMsgBoard();
}
// update MsgBoard
void AG_GUI::OnUpdateMsgBoard(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->emu->is_shown_message_board());
}

// Change UseJoypad
void AG_GUI::OnSelectUseJoypad(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->ToggleUseJoypad();
}
// update UseJoypad
void AG_GUI::OnUpdateUseJoypad(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->emu->is_enable_joypad());
}

// Change EnableLightpen
void AG_GUI::OnSelectEnableLightpen(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->emu->toggle_enable_lightpen();
}
// update EnableLightpen
void AG_GUI::OnUpdateEnableLightpen(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);

	AG_GUI_MENU_CHECK(mi, gui->emu->is_enable_lightpen());
}

// create keybind dialog
void AG_GUI::OnSelectKeybindBox(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->CreateKeybindBox();
}

// create config dialog
void AG_GUI::OnSelectConfigureBox(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	gui->CreateConfigureBox();
}

// Change Print Printer
void AG_GUI::OnSelectPrintPrinter(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->emu->print_printer(drv);
}
// update Print Printer
void AG_GUI::OnUpdatePrintPrinter(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	int size = gui->emu->get_printer_buffer_size(drv);

	AG_GUI_MENU_ENABLE(mi, size > 0);
}

// Change Direct Printer
void AG_GUI::OnSelectDirectPrinter(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->emu->enable_printer_direct(drv);
}
// update Direct Printer
void AG_GUI::OnUpdateDirectPrinter(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
//	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, config.printer_direct[drv]);
}

// Change Comm Server
void AG_GUI::OnSelectCommServer(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->emu->enable_comm_server(drv);
}
// update Comm Server
void AG_GUI::OnUpdateCommServer(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
//	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, config.comm_server[drv]);
}

// Change Comm connect
void AG_GUI::OnSelectCommConnect(AG_Event *event)
{
	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	gui->emu->enable_comm_connect(drv);
}
// update Comm connect
void AG_GUI::OnUpdateCommConnect(AG_Event *event)
{
	AG_MenuItem *mi = (AG_MenuItem *)AG_SENDER();
//	AG_GUI *gui = (AG_GUI *)AG_PTR(1);
	int drv = AG_INT(2);

	AG_GUI_MENU_CHECK(mi, config.comm_connect[drv]);
}
