/** @file logging.cpp

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.02.18 -

	@brief [ logging ]
*/

#include "logging.h"
#include "vm/vm.h"
#include "utility.h"

Logging::Logging()
{
	fplog = NULL;
}

Logging::~Logging()
{
}

void Logging::open_logfile(const _TCHAR *path)
{
#ifdef _LOG_FILE
	// get application path
	char log_path[_MAX_PATH];
	sprintf(log_path, "%s%s.log", UTILITY::cconv_from_native_path(path), CONFIG_NAME);

	fplog = fopen(log_path, "w");
#endif
}

void Logging::close_logfile()
{
#ifdef _LOG_FILE
	if(fplog) {
		fclose(fplog);
		fplog = NULL;
	}
#endif
}

void Logging::out_logv(int level, const _TCHAR* format, va_list ap)
{
	_TCHAR buffer[1024];
	_TCHAR levelstr[32];

	_vstprintf(buffer, format, ap);

	switch(level) {
	case LOG_WARN:
		_tcscpy(levelstr, _T("WARN"));
		break;
	case LOG_ERROR:
		_tcscpy(levelstr, _T("ERROR"));
		break;
	case LOG_DEBUG:
		_tcscpy(levelstr, _T("DEBUG"));
		break;
	default:
		_tcscpy(levelstr, _T("INFO"));
		break;
	}

#ifdef _LOG_CONSOLE
	_ftprintf(stderr, "%s: %s\n", levelstr, buffer);
	fflush(stderr);
#endif

	send_log_message(level, levelstr, buffer);

#ifdef _LOG_FILE
	if(fplog) {
		_ftprintf(fplog, _T("%10d %-5s: %s\n"), SDL_GetTicks(), levelstr, buffer);
		fflush(fplog);
	}
#endif
}

void Logging::out_log(int level, const _TCHAR* format, ...)
{
	va_list ap;

	va_start(ap, format);
	out_logv(level, format, ap);
	va_end(ap);
}

void Logging::out_debug(const _TCHAR* format, ...)
{
#ifdef _DEBUG_LOG
	va_list ap;

	va_start(ap, format);
	out_logv(LOG_DEBUG, format, ap);
	va_end(ap);
#endif
}
