/** @file board.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.06.21 -

	@brief [ main board ]
*/

#ifndef _BOARD_H_
#define _BOARD_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"
#include "../../config.h"

/**
	@brief main board -	process the RESET signal
*/
class BOARD : public DEVICE
{
private:
	outputs_t outputs_reset;

// for save config
#pragma pack(1)
	struct vm_state_st {
		int		fdd_type;
		int		io_port;
		uint8	flags;

		char  reserved[7];
	};
#pragma pack()

public:
	BOARD(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier) {
		set_class_name("BOARD");
		init_output_signals(&outputs_reset);
	}
	~BOARD() {}

	// common functions
	void write_signal(int id, uint32 data, uint32 mask);

	// unique functions
	void set_context_reset(DEVICE* device, int id, uint32 mask) {
		register_output_signal(&outputs_reset, device, id, mask);
	}

	void save_state(FILEIO *fio);
	void load_state(FILEIO *fio);
};

#endif

