/** @file keyboard.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.08 -

	@brief [ keyboard ]
*/

#ifndef _KEYBOARD_H_
#define _KEYBOARD_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"
#include "../../config.h"

#define SIG_KEYBOARD_HSYNC		1

#define ADDR_KB_NMI		0xffc8
#define ADDR_LP_FLG		0xffcb
#define ADDR_KEYBOARD	0xffe0
#define ADDR_LPEN_BL	0xffd5

/**
	@brief keyboard
*/
class KEYBOARD : public DEVICE
{
private:
	DEVICE *d_cpu,*d_disp,*d_board;

	uint8* key_stat;
	uint8  scan_code;
	uint8  key_scan_code;
//	uint8  key_native_code;
	uint8  key_pressed;

	uint8 kb_mode;
	uint8 kb_nmi;	// break key

	// light pen
	int*	mouse_stat;
	uint8	lpen_flg;
	uint8	lpen_flg_prev;
	bool	lpen_bl;

	uint32 *joy_stat;

	int	counter;	// keyboard counter
	int remain_count;

	int  *key_mod;
	bool pause_pressed;
	bool altkey_pressed;

	uint8 vm_key_stat[KEYBIND_KEYS];

//	int frame_counter;
//	int event_counter;

	uint32 scan2key_map[KEYBIND_KEYS][KEYBIND_ASSIGN];
	uint32 scan2key_preset_map[KEYBIND_PRESETS][KEYBIND_KEYS][KEYBIND_ASSIGN];
	uint32 scan2joy_map[KEYBIND_KEYS][KEYBIND_ASSIGN];
	uint32 scan2joy_preset_map[KEYBIND_PRESETS][KEYBIND_KEYS][KEYBIND_ASSIGN];
//	bool now_autokey;

	// key record
#ifdef USE_KEY_RECORD
	FILEIO *fkro, *fkri;
	uint8 vm_key_recp_stat[KEYBIND_KEYS];
	uint8 vm_key_recr_stat[KEYBIND_KEYS];
	int   mouse_recp_stat[3];
	int   mouse_recr_stat[3];
	char  rec_key_tmp_buff[512];
	char  rec_key_rec_buff[512];
	int64 key_rec_sum_clock;
#ifdef _DEBUG
	uint8 vm_key_dbg_stat[KEYBIND_KEYS];
#endif
#endif

	//for resume
#pragma pack(1)
	struct vm_state_st {
		uint8 kb_mode;
		uint8 kb_nmi;
		uint8 lpen_flg;
		uint8 lpen_flg_prev;
		uint8 lpen_bl;

		char  reserved1[3];

		int   counter;
		int   remain_count;
	};
#pragma pack()

	void clear_scan_code(int);
	void update_scan_code(int);
	void update_special_key();
	void update_light_pen();
	void update_keyboard();

	inline bool pressing_key(int);

	inline void reading_keys(int);
	inline void recording_keys(int code, bool pressed);
	inline void recording_system_keys(int code, int pressed);
	inline void recording_mouse_status();

	bool load_cfg_file();
	void save_cfg_file();
	bool load_ini_file();
	void save_ini_file();
public:
	KEYBOARD(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier)
	{
		set_class_name("KEYBOARD");
	}
	~KEYBOARD() {}

	// common functions
	void initialize();
	void reset();
	void release();
	void write_signal(int id, uint32 data, uint32 mask);
	void event_frame();

	void write_io8(uint32 addr, uint32 data);
	uint32 read_io8(uint32 addr);

	// unique functions
	void set_context_cpu(DEVICE* device) {
		d_cpu = device;
	}
	void set_context_disp(DEVICE* device) {
		d_disp = device;
	}
	void set_context_board(DEVICE* device) {
		d_board = device;
	}
	uint8 get_kb_mode() {
		return kb_mode;
	}
	void update_system_key();
	void system_key_down(int code);
	void system_key_up(int code);

	void event_callback(int event_id, int err);

	void save_keybind();

	void update_config();

#ifdef USE_KEY_RECORD
	bool play_reckey(_TCHAR* filename);
	bool record_reckey(_TCHAR* filename);
	void stop_reckey(bool stop_play = true, bool stop_record = true);
#endif

	void save_state(FILEIO *fio);
	void load_state(FILEIO *fio);

};

#endif

