/** @file keyboard_bind.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.04.08 -

	@brief [ keyboard bind ]
*/

#include "../vm.h"

#define VK_KEY_TYPE "SDL"

/* l3        -> common  win         linux mac          */
/* [caps]    -> F7      shift + capslock  capslock     */
/* [graph]   -> F8      muhenkan          rightcommand */
/* [kana]    -> tab     henkan            leftcommand  */
/* [break]   -> F9                                     */
/* [num ?]   -> end                                    */
/* [del  ]   ->         bs                delete       */
/* [num del] -> del                                    */
/* reset     ->         F11               F10          */
/* (pause)   ->         pause             F15          */
/* (alt)     ->         alt               option       */

static const uint32 scan2key_defmap[KEYBIND_KEYS][KEYBIND_ASSIGN]={
#if defined(__APPLE__) && defined(__MACH__)
//
// macosx
//
//	 space             up             num?            left             down             right             ctrl                    shift
//   0x80,             0x81,          0x82,           0x83,            0x84,            0x85,             0x86,                   0x87,
	{SDLK_SPACE,0x00},{SDLK_UP,0x00},{SDLK_END,0x00},{SDLK_LEFT,0x00},{SDLK_DOWN,0x00},{SDLK_RIGHT,0x00},{SDLK_LCTRL,SDLK_RCTRL},{SDLK_LSHIFT,SDLK_RSHIFT},
//	             caps                    kana                    graph                  esc                num8            num9            num*
//   0x88,       0x89,                   0x8a,                   0x8b,                  0x8c,              0x8d,           0x8e,           0x8f,
	{0x00,0x00},{SDLK_CAPSLOCK,SDLK_F7},{SDLK_LMETA,SDLK_TAB},{SDLK_RMETA,SDLK_F8},{SDLK_ESCAPE,0x00},{SDLK_KP8,0x00},{SDLK_KP9,0x00},{SDLK_KP_MULTIPLY,0x00},
//	 7             4             6             8             0             ^                 =-                3
//   0x90,         0x91,         0x92,         0x93,         0x94,         0x95,             0x96,             0x97
	{SDLK_7,0x00},{SDLK_4,0x00},{SDLK_6,0x00},{SDLK_8,0x00},{SDLK_0,0x00},{SDLK_CARET,0x00},{SDLK_MINUS,0x00},{SDLK_3,0x00},
//	 num ins/del         5            1             2             9             num7            ins/del              \
//   0x98,              0x99,         0x9a,         0x9b,         0x9c,         0x9d,           0x9e,                0x9f,
	{SDLK_DELETE,0x00},{SDLK_5,0x00},{SDLK_1,0x00},{SDLK_2,0x00},{SDLK_9,0x00},{SDLK_KP7,0x00},{SDLK_BACKSPACE,0x00},{SDLK_BACKSLASH,0x00},
//	 U             R             Y             I             P             [                       @              num0
//   0xa0,         0xa1,         0xa2,         0xa3,         0xa4,         0xa5,                   0xa6,          0xa7,
	{SDLK_u,0x00},{SDLK_r,0x00},{SDLK_y,0x00},{SDLK_i,0x00},{SDLK_p,0x00},{SDLK_LEFTBRACKET,0x00},{SDLK_AT,0x00},{SDLK_KP0,0x00},
//	 Q             T             W             E             O             num.                  home/cls         return
//   0xa8,         0xa9,         0xaa,         0xab,         0xac,         0xad,                 0xae,            0xaf,
	{SDLK_q,0x00},{SDLK_t,0x00},{SDLK_w,0x00},{SDLK_e,0x00},{SDLK_o,0x00},{SDLK_KP_PERIOD,0x00},{SDLK_HOME,0x00},{SDLK_KP_ENTER,SDLK_RETURN},
//   J             F             H             K             +;                    ]                        *:                num4
//   0xb0,         0xb1,         0xb2,         0xb3,         0xb4,                 0xb5,                    0xb6,             0xb7,
	{SDLK_j,0x00},{SDLK_f,0x00},{SDLK_h,0x00},{SDLK_k,0x00},{SDLK_SEMICOLON,0x00},{SDLK_RIGHTBRACKET,0x00},{SDLK_COLON,0x00},{SDLK_KP4,0x00},
//   A             G             S             D             L             num5            num6            num-
//   0xb8,         0xb9,         0xba,         0xbb,         0xbc,         0xbd,           0xbe,           0xbf,
	{SDLK_a,0x00},{SDLK_g,0x00},{SDLK_s,0x00},{SDLK_d,0x00},{SDLK_l,0x00},{SDLK_KP5,0x00},{SDLK_KP6,0x00},{SDLK_KP_MINUS,0x00},
//   M             V             N             <,                ?/                num/                  _                      num1
//   0xc0,         0xc1,         0xc2,         0xc3,             0xc4,             0xc5,                 0xc6,                  0xc7,
	{SDLK_m,0x00},{SDLK_v,0x00},{SDLK_n,0x00},{SDLK_COMMA,0x00},{SDLK_SLASH,0x00},{SDLK_KP_DIVIDE,0x00},{SDLK_UNDERSCORE,0x00},{SDLK_KP1,0x00},
//   Z             B             X             C             >.                 num2            num3            num+
//   0xc8,         0xc9,         0xca,         0xcb,         0xcc,              0xcd,           0xce,           0xcf,
	{SDLK_z,0x00},{SDLK_b,0x00},{SDLK_x,0x00},{SDLK_c,0x00},{SDLK_PERIOD,0x00},{SDLK_KP2,0x00},{SDLK_KP3,0x00},{SDLK_KP_PLUS,0x00},
//   F1             F2             F3             F4             F5
//   0xd0,          0xd1,          0xd2,          0xd3,          0xd4,          0xd5,       0xd6,       0xd7,
	{SDLK_F1,0x00},{SDLK_F2,0x00},{SDLK_F3,0x00},{SDLK_F4,0x00},{SDLK_F5,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},

	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   (pause)        (alt)
	{SDLK_F15,0x00},{SDLK_LALT,SDLK_RALT},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//                                                                                       reset
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{SDLK_F10,0x00},
//   break         (property)
	{SDLK_F9,0x00},{0x00,0x00}

#else
//
// default map
//
//	 space             up             num?            left             down             right             ctrl                    shift
//   0x80,             0x81,          0x82,           0x83,            0x84,            0x85,             0x86,                   0x87,
	{SDLK_SPACE,0x00},{SDLK_UP,0x00},{SDLK_END,0x00},{SDLK_LEFT,0x00},{SDLK_DOWN,0x00},{SDLK_RIGHT,0x00},{SDLK_LCTRL,SDLK_RCTRL},{SDLK_LSHIFT,SDLK_RSHIFT},
//	             caps                    kana                    graph                  esc                num8            num9            num*
//   0x88,       0x89,                   0x8a,                   0x8b,                  0x8c,              0x8d,           0x8e,           0x8f,
	{0x00,0x00},{SDLK_CAPSLOCK,SDLK_F7},{SDLK_WORLD_0,SDLK_TAB},{SDLK_WORLD_1,SDLK_F8},{SDLK_ESCAPE,0x00},{SDLK_KP8,0x00},{SDLK_KP9,0x00},{SDLK_KP_MULTIPLY,0x00},
//	 7             4             6             8             0             ^                 =-                3
//   0x90,         0x91,         0x92,         0x93,         0x94,         0x95,             0x96,             0x97
	{SDLK_7,0x00},{SDLK_4,0x00},{SDLK_6,0x00},{SDLK_8,0x00},{SDLK_0,0x00},{SDLK_CARET,0x00},{SDLK_MINUS,0x00},{SDLK_3,0x00},
//	 num ins/del         5            1             2             9             num7            ins/del              \
//   0x98,              0x99,         0x9a,         0x9b,         0x9c,         0x9d,           0x9e,                0x9f,
	{SDLK_DELETE,0x00},{SDLK_5,0x00},{SDLK_1,0x00},{SDLK_2,0x00},{SDLK_9,0x00},{SDLK_KP7,0x00},{SDLK_BACKSPACE,0x00},{SDLK_BACKSLASH,0x00},
//	 U             R             Y             I             P             [                       @              num0
//   0xa0,         0xa1,         0xa2,         0xa3,         0xa4,         0xa5,                   0xa6,          0xa7,
	{SDLK_u,0x00},{SDLK_r,0x00},{SDLK_y,0x00},{SDLK_i,0x00},{SDLK_p,0x00},{SDLK_LEFTBRACKET,0x00},{SDLK_AT,0x00},{SDLK_KP0,0x00},
//	 Q             T             W             E             O             num.                  home/cls         return
//   0xa8,         0xa9,         0xaa,         0xab,         0xac,         0xad,                 0xae,            0xaf,
	{SDLK_q,0x00},{SDLK_t,0x00},{SDLK_w,0x00},{SDLK_e,0x00},{SDLK_o,0x00},{SDLK_KP_PERIOD,0x00},{SDLK_HOME,0x00},{SDLK_KP_ENTER,SDLK_RETURN},
//   J             F             H             K             +;                    ]                        *:                num4
//   0xb0,         0xb1,         0xb2,         0xb3,         0xb4,                 0xb5,                    0xb6,             0xb7,
	{SDLK_j,0x00},{SDLK_f,0x00},{SDLK_h,0x00},{SDLK_k,0x00},{SDLK_SEMICOLON,0x00},{SDLK_RIGHTBRACKET,0x00},{SDLK_COLON,0x00},{SDLK_KP4,0x00},
//   A             G             S             D             L             num5            num6            num-
//   0xb8,         0xb9,         0xba,         0xbb,         0xbc,         0xbd,           0xbe,           0xbf,
	{SDLK_a,0x00},{SDLK_g,0x00},{SDLK_s,0x00},{SDLK_d,0x00},{SDLK_l,0x00},{SDLK_KP5,0x00},{SDLK_KP6,0x00},{SDLK_KP_MINUS,0x00},
//   M             V             N             <,                ?/                num/                  _                      num1
//   0xc0,         0xc1,         0xc2,         0xc3,             0xc4,             0xc5,                 0xc6,                  0xc7,
	{SDLK_m,0x00},{SDLK_v,0x00},{SDLK_n,0x00},{SDLK_COMMA,0x00},{SDLK_SLASH,0x00},{SDLK_KP_DIVIDE,0x00},{SDLK_UNDERSCORE,0x00},{SDLK_KP1,0x00},
//   Z             B             X             C             >.                 num2            num3            num+
//   0xc8,         0xc9,         0xca,         0xcb,         0xcc,              0xcd,           0xce,           0xcf,
	{SDLK_z,0x00},{SDLK_b,0x00},{SDLK_x,0x00},{SDLK_c,0x00},{SDLK_PERIOD,0x00},{SDLK_KP2,0x00},{SDLK_KP3,0x00},{SDLK_KP_PLUS,0x00},
//   F1             F2             F3             F4             F5
//   0xd0,          0xd1,          0xd2,          0xd3,          0xd4,          0xd5,       0xd6,       0xd7,
	{SDLK_F1,0x00},{SDLK_F2,0x00},{SDLK_F3,0x00},{SDLK_F4,0x00},{SDLK_F5,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},

	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   (pause)          (alt)
	{SDLK_PAUSE,0x00},{SDLK_LALT,SDLK_RALT},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//                                                                                       reset
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{SDLK_F11,0x00},
//   break         (property)
	{SDLK_F9,0x00},{0x00,0x00}
#endif
};

static const uint32 scan2joy_defmap[KEYBIND_KEYS][KEYBIND_ASSIGN]={
//	 space       up          num?        left        down        right       ctrl        shift
//   0x80,       0x81,       0x82,       0x83,       0x84,       0x85,       0x86,       0x87,
	{0x10,0x00},{0x01,0x00},{0x00,0x00},{0x04,0x00},{0x02,0x00},{0x08,0x00},{0x00,0x00},{0x00,0x00},
//	             caps        kana        graph       esc         num8        num9        num*
//   0x88,       0x89,       0x8a,       0x8b,       0x8c,       0x8d,       0x8e,       0x8f,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x01},{0x00,0x09},{0x00,0x00},
//	 7           4           6           8           0           ^           =-          3
//   0x90,       0x91,       0x92,       0x93,       0x94,       0x95,       0x96,       0x97
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//	 num ins/del 5           1           2           9           num7        ins/del     \
//   0x98,       0x99,       0x9a,       0x9b,       0x9c,       0x9d,       0x9e,       0x9f,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x05},{0x00,0x00},{0x00,0x00},
//	 U           R           Y           I           P           [           @           num0
//   0xa0,       0xa1,       0xa2,       0xa3,       0xa4,       0xa5,       0xa6,       0xa7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x10},
//	 Q           T           W           E           O           num.        home/cls    return
//   0xa8,       0xa9,       0xaa,       0xab,       0xac,       0xad,       0xae,       0xaf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   J           F           H           K           +;          ]           *:          num4
//   0xb0,       0xb1,       0xb2,       0xb3,       0xb4,       0xb5,       0xb6,       0xb7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x04},
//   A           G           S           D           L           num5        num6        num-
//   0xb8,       0xb9,       0xba,       0xbb,       0xbc,       0xbd,       0xbe,       0xbf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x08},{0x00,0x00},
//   M           V           N           <,          ?/          num/        _           num1
//   0xc0,       0xc1,       0xc2,       0xc3,       0xc4,       0xc5,       0xc6,       0xc7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x06},
//   Z           B           X           C           >.          num2        num3        num+
//   0xc8,       0xc9,       0xca,       0xcb,       0xcc,       0xcd,       0xce,       0xcf,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x02},{0x00,0x0a},{0x00,0x00},
//   F1          F2          F3          F4          F5
//   0xd0,       0xd1,       0xd2,       0xd3,       0xd4,       0xd5,       0xd6,       0xd7,
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},

	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   (pause)
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//                                                                                       reset
	{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},{0x00,0x00},
//   break      (property)
	{0x00,0x00},{0x00,0x00}
};

static const uint16 kb2scan_map[128] = {
//  0    1    2    3    4    5    6    7    8    9    a    b    c    d    e    f
	0xff,0x80,0x7f,0xff,0x60,0x61,0x1e,0x18,0xff,0xff,0xff,0xff,0x2e,0x2f,0xff,0xff,
	0x07,0x06,0xff,0xff,0x09,0x0a,0x0b,0xff,0xff,0xff,0xff,0x0c,0x05,0x03,0x01,0x04,
	0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
	0x14,0x1a,0x1b,0x17,0x11,0x19,0x12,0x10,0x13,0x1c,0x36,0x34,0x43,0x16,0x4c,0x44,
	0x26,0x38,0x49,0x4b,0x3b,0x2b,0x31,0x39,0x32,0x23,0x30,0x33,0x3c,0x40,0x42,0x2c,
	0x24,0x28,0x21,0x3a,0x29,0x20,0x41,0x2a,0x4a,0x22,0x48,0x25,0x1f,0x35,0x15,0x46,
	0x27,0x47,0x4d,0x4e,0x37,0x3d,0x3e,0x1d,0x0d,0x0e,0x0f,0x4f,0x02,0x3f,0x2d,0x45,
	0x50,0x51,0x52,0x53,0x54,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
};

